var path = require('path');

module.exports = {
  entry: './src/index.js',
  output: {      
    path: path.join(__dirname, './dist'),      
    filename: 'index.js',      
    library: 'library',      
    libraryTarget: 'umd',      
    publicPath: '/dist/',      
    umdNamedDefine: true  
  },
  module: {
    rules: [
      {
        test: /\.(css)$/, 
        loader: 'css-loader'
      },
      {
        test: /\.(eot|ttf|woff|woff2)$/,
        loader: 'file-loader'
      },
      {
        test: /\.(js|jsx)$/,
        exclude: /(node_modules|bower_components|build)/,
        use: [
          {
            loader: 'babel-loader',
            query: {
              presets : ['es2015', 'react', 'stage-0'],
              plugins: [
                "transform-class-properties",
                "transform-object-rest-spread",
                "transform-react-jsx",
                ["transform-imports", {
                  "react-bootstrap": {
                    "transform": "react-bootstrap/lib/${member}",
                    "preventFullImport": true
                  }
                }]
            ],
            }
          },
          {
            loader: 'eslint-loader'
          }
        ],
      },
      {
        test: /\.(png|jpg|gif|svg)$/,
        use: [
          {
            loader: 'url-loader',
            options: {
              fallback: 'responsive-loader',
              quality: 85
            }
          }
        ]
      }
    ]
  },
  resolve: {      
    alias: {          
      'react': path.resolve(__dirname, './node_modules/react'),
      'react-dom': path.resolve(__dirname, './node_modules/react-dom'),      
    }  
  },  
  externals: {      
    react: {          
        commonjs: "react",          
        commonjs2: "react",          
        amd: "React",          
        root: "React"      
    },      
    "react-dom": {          
        commonjs: "react-dom",          
        commonjs2: "react-dom",          
        amd: "ReactDOM",          
        root: "ReactDOM"      
    }  
  }
};